<?php
/**
 * @file
 * Functions to manipulate xc_search_attribute_to_field table.
 * The table's structure:
 * CREATE TABLE xc_search_attribute_to_field (
 *   atof_id         int(10) unsigned NOT NULL auto_increment,
 *   metadata_schema varchar(255) NOT NULL default '',
 *   entity          varchar(255) NOT NULL default '',
 *   schema_field    varchar(255) NOT NULL default '',
 *   attribute       varchar(255) NOT NULL default '',
 *   type            varchar(20)  NOT NULL default '',
 *   solr_field      varchar(255) NOT NULL default '',
 *   PRIMARY KEY  (`atof_id`),
 *   KEY `field_type_id` (`atof_id`)
 * );
 */


function xc_search_attribute_metadata_load($entity_name) {
  if (!isset($entity_name) || trim($entity_name) == "") {
    return NULL;
  }
  $schemas = module_invoke_all('xc_schema');
  $schema = $schemas[$entity_name];
  $schema['entity_name'] = $entity_name;
  $schema['metadata_schema'] = xc_format_get_schema_by_entity($entity_name);
  return $schema;
}

function xc_search_attribute_view() {
  $output = '<p>' . t('Select a metadata record type (entity) to list and modify 
    how to index its fields\' attributes.') . '</p>';
  
  $items = array();
  $schemas = module_invoke_all('xc_schema');
  foreach ($schemas as $key => $value) {
    $items[] = l($value['title'], 'xc/search/indexing/attribute/' . $key);
  }
  $output .= theme('item_list', $items, t('Available record types:'));
  return $output;
}

/**
 * Return the array of array of the attributes and its properties. The first
 * key of the array is the field name, the second key is the attribute name
 * (without the namespace part if $do_namespace_hack argument is TRUE). The value
 * of the key is an associative array with 3 fields: id, type (ignore, 
 * value_as_key, as_key_exclusively, value_as_value) and solr field name.
 * @param $do_namespace_hack If true we remove the namespace part from attribute name.
 * @return The array of attributes
 */
function xc_search_attribute_get_attributes($do_namespace_hack = FALSE) {
  $sql = "SELECT * FROM {xc_search_attribute_to_field}";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_attribute_to_field}', 
    'atof_id'));
  $values = array();
  while ($data = db_fetch_object($result)) {
    if ($do_namespace_hack == TRUE && module_exists('xc_util')) {
      $data->attribute = xc_util_remove_ns($data->attribute);
    }
    $values[$data->schema_field][$data->attribute] = array(
      'id' => $data->atof_id, 
      'type' => $data->type, 
      'field' => $data->solr_field,
    );
  }
  return $values;
}

function xc_search_attribute_get_solr_fields($metadata_schema) {
  $sql = "SELECT schema_field, attribute, solr_field FROM {xc_search_attribute_to_field} 
           WHERE type != 'ignore'
             AND metadata_schema = '%s'";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_attribute_to_field}', 
    'atof_id'), $metadata_schema);
  $values = array();
  while ($data = db_fetch_object($result)) {
    $values[$data->solr_field] = $data->schema_field . '@' . $data->attribute;
  }
  return $values;
}

function xc_search_attribute_form(&$form_state, $metadata_schema) {
  drupal_set_title(t('Handling %entity entity\'s fields\' attributes', 
    array('%entity' => $metadata_schema['entity_name'])));

  $existing_attributes = xc_search_attribute_get_all_by_schema(
    $metadata_schema['metadata_schema'], 
    $metadata_schema['entity_name']
  );
  
  //$schema = module_invoke($schema_name, 'xc_schema');
  $types = xc_search_attribute_get_handler_types();
  foreach ($metadata_schema['fields'] as $field => $obj) {
    if (isset($obj['attributes'])) {
      foreach ($obj['attributes'] as $attribute => $attrObject) {
        $default_value = 'ignore';
        $default_exclusive = 0;
        if (isset($existing_attributes[$field]) 
          && isset($existing_attributes[$field][$attribute])) {
          $default_value = $existing_attributes[$field][$attribute]->type;
        }
        $form['types'][$field . '---' . $attribute] = array(
          '#type' => 'radios',
          '#options' => $types,
          '#default_value' => $default_value,
        );
      }
    }
  }
  
  $form['metadata_schema'] = array(
    '#type' => 'hidden',
    '#value' => $metadata_schema['metadata_schema'],
  );
  $form['entity_name'] = array(
    '#type' => 'hidden',
    '#value' => $metadata_schema['entity_name'],
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['restore'] = array(
    '#type' => 'submit',
    '#value' => t('Restore defaults'),
  );
  $form['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
  );
  
  return $form;
}

function theme_xc_search_attribute_form($form) {
  $types = xc_search_attribute_get_handler_types();
  $output = '<p>' . theme('image', 'http://wiki.apache.org/moin_static184/modernized/img/alert.png', 
    t('Alert!'), t('Alert!'), NULL, FALSE) . ' ' . t('Alert! Advanced settings! Modify these settings 
    very carefully, and only after you have read the manual!') . '</p>';
  $output .= t('We can handle the attributes in different ways:');
  $output .= theme('item_list', array_values($types));
  
  $output .= '<p>' . t('<strong>ignore</strong> means, that we do not index that attribute at all.') . '</p>';
  $output .= '<p>' . t('Otherwise we will index the attribute somehow. But there are some case,
    when the the attribute somehow qualifies the parent elements name, and other cases, when it
    contains brand new informations. An example for the first case is a subject field, where the
    type attribute localize the source of the subject term. An exemple for the second case is
    when an author record contain not only the name of the author, but the author\'s ID in
    an attibute.') . '</p>';
  $output .= '<p>' . t('<strong>as key</strong> means, that the attribute\'s value will be the part of the field,
    and we value of the parent element will be the value of this new field.Imagine this: 
    &lt;subject type="OCLC"&gt;History&lt;/subject&gt;. It means, that the History term comes from
    OCLC subject headings. Here the value could be use as key, and we can store the term History
    as subject, and as OCLC subject. The Drupal Toolkit will create a new field for this, joining
    the field name and the attribute name together, like this: subject_OCLC, so finally we will have 
    two fields: subject = "History" and subject_OCLC = "History".') . '</p>';
  $output .= '<p>' . t('<strong>as key excusively</strong> means, that when the <em>as key</em> option was selected
    only the new field (created from the attribute value) is indexed, and not the general main one.
    So &mdash; if we continue the previous example &mdash; only subject_OCLC = "History" will be indexed, 
    and not the general subject = "History".') . '</p>';
  $output .= '<p>' . t('<strong>as value</strong> means the second case, we use the attribute\'s name as the part of the
    new field name, and its value as the value of that field (and not the element\'s value).
    Example: &lt;author agentID="123"&gt;Shakespeare&lt;/subject&gt; becames author = "Shakespeare"
    and author_agentID = "123".') . '</p>';
  $output .= '<p>' . t('You can map the field types created from attrubutes in the !search_field_type 
    form. Those fields which still has not mapped to a type are displayed 
    in <font color="red">red</font>.',
    array('!search_field_type' => l('field types', 'xc/search/indexing/field_type'))) . '</p>';
  
  $metadata_schema = $form['metadata_schema']['#value'];
  $entity_name     = $form['entity_name']['#value'];
  $existing_attributes = xc_search_attribute_get_all_by_schema($metadata_schema, $entity_name);
  
  $rows = array();
  $row = array();
  $prev_field = "";
  foreach (element_children($form['types']) as $key) {
    list($field, $attribute) = split('---', $key);
    $row = array($field);
    $row[] = isset($existing_attributes[$field]) 
      ? $attribute 
      : array('data' => $attribute, 'style' => 'color:red');
    foreach (element_children($form['types'][$key]) as $type) {
      $row[] = array('data' => drupal_render($form['types'][$key][$type]));
    }
    
    $rows[] = $row;
    $prev_field = $field;
  }
  
  $header = array(t('field'), t('attribute'));
  foreach ($types as $suffix => $type) {
    $header[] = $type;
  }
  
  $output .= theme('table', $header, $rows);
  $output .= drupal_render($form);
  return $output;
}

function xc_search_attribute_form_submit($form, &$form_state) {
  $op = isset($form_state['values']['op']) ? $form_state['values']['op'] : ''; 
  $values      = $form_state['values'];
  $entity_name = $values['entity_name'];
  
  if ($op == t('Save')) {
    xc_search_attribute_save_form($values);
  }
  elseif ($op == t('Restore defaults')) {
    xc_search_attribute_restore_defaults($values);
  }
  
  $form_state['redirect'] = 'xc/search/indexing/attribute/' . $entity_name;
}

/**
 * Restore default values for an entities' attribute handling
 * @param $values
 * @return unknown_type
 */
function xc_search_attribute_restore_defaults($values) {
  $metadata_schema = $values['metadata_schema'];
  $entity_name     = $values['entity_name'];
  $sql = "DELETE FROM {xc_search_attribute_to_field}
           WHERE metadata_schema = '%s'
             AND entity = '%s'";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_attribute_to_field}', 
    'atof_id'), $metadata_schema, $entity_name);
  if ($result == SAVED_DELETED) { // attribute is deleted
    drupal_set_message(t('All attribute removed from %schema/%entity', 
      array('%schema' => $metadata_schema, '%entity' => $entity_name)));
    $data = xc_search_attribute_get_raw_defaults();
    foreach ($data as $record) {
      if ($record->metadata_schema == $metadata_schema && $record->entity == $entity_name) {
        $ret_val = drupal_write_record('xc_search_attribute_to_field', $record);
        if ($ret_val == SAVED_NEW) {
          drupal_set_message(t('Successfully restred %mt/%ent/%field@%attr as %type type.',
            array('%mt' => $metadata_schema, '%ent' => $entity_name,
            '%field' => $record->schema_field, '%attr' => $record->attribute, '%type' => $record->type)));
        } 
        else {
          drupal_set_message(t('Unexpected error. Failed to register attribute mapping.'));
        }
      }
    }
  }
  else {
    drupal_set_message(t('Unexpected error. Failed to remove attribute handling.'));
  }
}
  
/**
 * Save the corrent values to database
 * @param $values
 * @return unknown_type
 */
function xc_search_attribute_save_form($values) {
  $metadata_schema = $values['metadata_schema'];
  $entity_name     = $values['entity_name'];
  $existing_attributes = xc_search_attribute_get_all_by_schema($metadata_schema, $entity_name);
  $handling_types = xc_search_attribute_get_handler_types();
  
  // build a same array from user data
  $actual_attributes = array();
  foreach ($values as $key => $value) {
    if ($value != '0' && preg_match('/^(.*?)---(.*?)$/', $key, $matches)) {
      $actual_attributes[$matches[1]][$matches[2]] = $value;
    }
  }
  
  foreach ($existing_attributes as $fieldName => $field) {
    foreach ($field as $attributeName => $data) {
      if (!isset($actual_attributes[$fieldName][$attributeName])) {
        $sql = "DELETE FROM {xc_search_attribute_to_field}
                 WHERE metadata_schema = '%s'
                   AND entity = '%s'
                   AND schema_field = '%s'
                   AND attribute = '%s'";
        $result = db_query(db_rewrite_sql($sql, '{xc_search_attribute_to_field}', 
          'atof_id'), $metadata_schema, $entity_name, $fieldName, $attributeName);
        if ($result == SAVED_DELETED) { // repository is deleted
          drupal_set_message(t('%field@%attribute removed', 
            array('%field' => $fieldName, '%attribute' => $attributeName)));
          // TODO: fieldtype: delete
        }
        else {
          drupal_set_message(t('Unexpected error. Failed to remove Solr field type.'));
        }
      }
    }
  }

  foreach ($actual_attributes as $fieldName => $field) {
    foreach ($field as $attributeName => $type) {
      if (isset($existing_attributes[$fieldName][$attributeName])) {
        if ($existing_attributes[$fieldName][$attributeName]->type == $type) {
          continue;
        }
        $record = new stdClass();
        $record->atof_id = $existing_attributes[$fieldName][$attributeName]->atof_id;
        $record->type = $type;
        if ($record->type == 'ignore') {
          $record->solr_field = "";
        }
        elseif ($record->type == 'value_as_key' || $record->type == 'as_key_exclusively') {
          $record->solr_field = $fieldName . '_xxxxxx';
        }
        else {
          $record->solr_field = $fieldName . '_' . $attributeName;
        }
        $ret_val = drupal_write_record('xc_search_attribute_to_field', $record, 'atof_id');
        if ($ret_val == SAVED_UPDATED) {
          drupal_set_message(t('Successfully updated %field@%attr facet as %type type.',
            array('%field' => $fieldName, '%attr' => $attributeName, '%type' => $record->type)));
        }
        else {
          drupal_set_message(t('Unexpected error. Failed to update attribute mapping.'));
        }
      }
      else { // new record should be created
        $record = new stdClass();
        $record->metadata_schema = $metadata_schema;
        $record->entity          = $entity_name;
        $record->schema_field    = $fieldName;
        $record->attribute       = $attributeName;
        $record->type            = $type;
        if ($record->type == 'ignore') {
          $record->solr_field = "";
        }
        elseif ($record->type == 'value_as_key' || $record->type == 'as_key_exclusively') {
          $record->solr_field = $fieldName . '_xxxxxx';
        }
        else {
          $record->solr_field = $fieldName . '_' . $attributeName;
        }
        //$record->solr_field      = $fieldName . '_' . $attributeName;
        $ret_val = drupal_write_record('xc_search_attribute_to_field', $record);
        if ($ret_val == SAVED_NEW) {
          drupal_set_message(t('Successfully registered %mt/%ent/%field@%attr facet as %type type.',
            array('%mt' => $metadata_schema, '%ent' => $entity_name,
            '%field' => $fieldName, '%attr' => $attributeName, '%type' => $record->type)));
        } 
        else {
          drupal_set_message(t('Unexpected error. Failed to register attribute mapping.'));
        }
      }
    }
  }
}

/**
 * Get the list of field-type mappings in the form of an array. 
 * 
 * Each element of the array
 * has an 'id' and a 'type' element. The keys are the field names.
 * 
 */
function xc_search_attribute_get_all_by_schema($metadata_schema, $entity_name) {
  $sql = "SELECT * FROM {xc_search_attribute_to_field} 
           WHERE metadata_schema = '%s' 
             AND entity = '%s'";
  $result = db_query(db_rewrite_sql($sql, '{xc_search_attribute_to_field}', 
    'atof_id'), $metadata_schema, $entity_name);
  $ret_val = array();
  while ($data = db_fetch_object($result)) {
    $ret_val[$data->schema_field][$data->attribute] = $data;
  }
  return $ret_val;
}

function xc_search_attribute_get_handler_types() {
  $types = array(
    'ignore'         => t('ignore'),
    'value_as_key'   => t('as key'), 
    'as_key_exclusively' => t('as key exclusively'), 
    'value_as_value' => t('as value')
  );
  
  return $types;
}

function xc_search_attribute_default() {
  $defaults = <<<END
metadata_schema;entity;schema_field;attribute;type;solr_field
xc;work;dcterms:coverage;xsi:type;value_as_key;dcterms:coverage_xxxxxx
xc;work;dcterms:isReplacedBy;dcterms:ISSN;value_as_value;dcterms:isReplacedBy_dcterms:ISSN
xc;work;dcterms:replaces;dcterms:ISSN;value_as_value;dcterms:replaces_dcterms:ISSN
xc;work;dcterms:spatial;xsi:type;value_as_key;dcterms:spatial_xxxxxx
xc;work;dcterms:subject;xsi:type;value_as_key;dcterms:subject_xxxxxx
xc;work;dcterms:type;xsi:type;value_as_key;dcterms:type_xxxxxx
xc;work;rdarole:artist;agentID;value_as_value;rdarole:artist_agentID
xc;work;rdarole:author;agentID;value_as_value;rdarole:author_agentID
xc;work;rdarole:compiler;agentID;value_as_value;rdarole:compiler_agentID
xc;work;rdarole:composer;agentID;value_as_value;rdarole:composer_agentID
xc;work;rdarole:speaker;agentID;value_as_value;rdarole:speaker_agentID
xc;work;rdvocab:identifierForTheWork;xsi:type;value_as_key;rdvocab:identifierForTheWork_xxxxxx
xc;work;xc:coverage;xsi:type;value_as_key;xc:coverage_xxxxxx
xc;work;xc:coverage;geoID;value_as_value;xc:coverage_geoID
xc;work;xc:creator;agentID;value_as_value;xc:creator_agentID
xc;work;xc:expressionOfWork;type;value_as_key;xc:expressionOfWork_xxxxxx
xc;work;xc:relation;workID;value_as_value;xc:relation_workID
xc;work;xc:spatial;xsi:type;value_as_key;xc:spatial_xxxxxx
xc;work;xc:spatial;geoID;value_as_value;xc:spatial_geoID
xc;work;xc:subject;xsi:type;value_as_key;xc:subject_xxxxxx
xc;work;xc:subject;subjID;value_as_value;xc:subject_subjID
xc;work;xc:temporal;type;value_as_key;xc:temporal_xxxxxx
xc;work;xc:temporal;chronID;value_as_value;xc:temporal_chronID
xc;work;xc:type;xsi:type;value_as_key;xc:type_xxxxxx
xc;work;xc:type;subjID;value_as_value;xc:type_subjID
xc;expression;dcterms:language;xsi:type;value_as_key;dcterms:language_xxxxxx
xc;expression;dcterms:type;xsi:type;value_as_key;dcterms:type_xxxxxx
xc;expression;rdarole:director;agentID;value_as_value;rdarole:director_agentID
xc;expression;rdarole:editor;agentID;value_as_value;rdarole:editor_agentID
xc;expression;rdarole:illustrator;agentID;value_as_value;rdarole:illustrator_agentID
xc;expression;rdarole:performer;agentID;value_as_value;rdarole:performer_agentID
xc;expression;rdarole:producer;agentID;value_as_value;rdarole:producer_agentID
xc;expression;rdarole:translator;agentID;value_as_value;rdarole:translator_agentID
xc;expression;rdvocab:identifierForTheExpression;type;value_as_key;rdvocab:identifierForTheExpression_xxxxxx
xc;expression;xc:contributor;agentID;value_as_value;xc:contributor_agentID
xc;expression;xc:manifestationOfExpression;type;value_as_key;xc:manifestationOfExpression_xxxxxx
xc;expression;xc:workExpressed;type;value_as_key;xc:workExpressed_xxxxxx
xc;manifestation;xc:expressionManifested;type;value_as_key;xc:expressionManifested_xxxxxx
xc;manifestation;xc:holdingsOfManifestation;type;value_as_key;xc:holdingsOfManifestation_xxxxxx
xc;manifestation;xc:identifier;xsi:type;value_as_key;xc:identifier_xxxxxx
xc;manifestation;xc:isPartOf;workID;value_as_value;xc:isPartOf_workID
xc;manifestation;xc:recordID;type;value_as_key;xc:recordID_xxxxxx
xc;holdings;xc:itemOfHoldings;type;value_as_key;xc:itemOfHoldings_xxxxxx
xc;holdings;xc:manifestationHeld;type;value_as_key;xc:manifestationHeld_xxxxxx
xc;holdings;xc:recordID;type;value_as_key;xc:recordID_xxxxxx
xc;item;xc:HoldingsExemplified;type;value_as_key;xc:HoldingsExemplified_xxxxxx
xc;item;xc:recordID;type;value_as_key;xc:recordID_xxxxxx
xc;item;xc:serviceProvider;type;value_as_key;xc:serviceProvider_xxxxxx
END;

  return $defaults;
}

/**
 * Return an array of stdClass of xc_search_attribute_to_fields records
 * @return unknown_type
 */
function xc_search_attribute_get_raw_defaults() {
  return xc_util_create_bulk_records(xc_search_attribute_default());
}